@@ -42,7 +42,10 @@ typedef struct GPBCodedInputStreamState {
 
 // Reads a group field value from the stream and merges it into the given
 // UnknownFieldSet.
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 - (void)readUnknownGroup:(int32_t)fieldNumber message:(GPBUnknownFieldSet *)message;
+#pragma clang diagnostic pop
 
 // Reads a map entry.
 - (void)readMapEntry:(id)mapDictionary
@@ -53,6 +56,7 @@ typedef struct GPBCodedInputStreamState {
 
 CF_EXTERN_C_BEGIN
 
+void GPBRaiseStreamError(NSInteger code, NSString *reason);
 int32_t GPBCodedInputStreamReadTag(GPBCodedInputStreamState *state);
 
 double GPBCodedInputStreamReadDouble(GPBCodedInputStreamState *state);
@@ -75,6 +79,9 @@ NSData *GPBCodedInputStreamReadRetainedBytes(GPBCodedInputStreamState *state)
     __attribute((ns_returns_retained));
 NSData *GPBCodedInputStreamReadRetainedBytesNoCopy(GPBCodedInputStreamState *state)
     __attribute((ns_returns_retained));
+NSData *GPBCodedInputStreamReadRetainedBytesToEndGroupNoCopy(GPBCodedInputStreamState *state,
+                                                             int32_t fieldNumber)
+    __attribute((ns_returns_retained));
 
 size_t GPBCodedInputStreamPushLimit(GPBCodedInputStreamState *state, size_t byteLimit);
 void GPBCodedInputStreamPopLimit(GPBCodedInputStreamState *state, size_t oldLimit);
