@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #import "GPBCodedOutputStream_PackagePrivate.h"
 
@@ -36,6 +13,10 @@
 #import "GPBUnknownFieldSet_PackagePrivate.h"
 #import "GPBUtilities_PackagePrivate.h"
 
+// TODO: Consider using on other functions to reduce bloat when
+// some compiler optimizations are enabled.
+#define GPB_NOINLINE __attribute__((noinline))
+
 // These values are the existing values so as not to break any code that might
 // have already been inspecting them when they weren't documented/exposed.
 NSString *const GPBCodedOutputStreamException_OutOfSpace = @"OutOfSpace";
@@ -60,6 +41,37 @@ typedef struct GPBOutputBufferState {
 static const int32_t LITTLE_ENDIAN_32_SIZE = sizeof(uint32_t);
 static const int32_t LITTLE_ENDIAN_64_SIZE = sizeof(uint64_t);
 
+// Helper to write bytes to an NSOutputStream looping in case a subset is written in
+// any of the attempts.
+GPB_NOINLINE
+static NSInteger WriteToOutputStream(NSOutputStream *output, uint8_t *bytes, size_t length) {
+  size_t total = 0;
+
+  while (length) {
+    NSInteger written = [output write:bytes maxLength:length];
+
+    // Fast path - done.
+    if (written == (NSInteger)length) {
+      return total + written;
+    }
+
+    if (written > 0) {
+      // Record the subset written and continue in case it was a partial write.
+      total += written;
+      length -= written;
+      bytes += written;
+    } else if (written == 0) {
+      // Stream refused to write more, return what was written.
+      return total;
+    } else {
+      // Return the error.
+      return written;
+    }
+  }
+
+  return total;
+}
+
 // Internal helper that writes the current buffer to the output. The
 // buffer position is reset to its initial value when this returns.
 static void GPBRefreshBuffer(GPBOutputBufferState *state) {
@@ -68,7 +80,7 @@ static void GPBRefreshBuffer(GPBOutputBufferState *state) {
     [NSException raise:GPBCodedOutputStreamException_OutOfSpace format:@""];
   }
   if (state->position != 0) {
-    NSInteger written = [state->output write:state->bytes maxLength:state->position];
+    NSInteger written = WriteToOutputStream(state->output, state->bytes, state->position);
     if (written != (NSInteger)state->position) {
       [NSException raise:GPBCodedOutputStreamException_WriteFailed format:@""];
     }
@@ -148,7 +160,14 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
 }
 
 - (void)dealloc {
-  [self flush];
+  @try {
+    [self flush];
+  } @catch (NSException *exception) {
+    // -dealloc methods cannot fail, so swallow any exceptions from flushing.
+#if defined(DEBUG) && DEBUG
+    NSLog(@"GPBCodedOutputStream: Exception while flushing in dealloc: %@", exception);
+#endif
+  }
   [state_.output close];
   [state_.output release];
   [buffer_ release];
@@ -333,6 +352,9 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
   [self writeGroupNoTag:fieldNumber value:value];
 }
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+
 - (void)writeUnknownGroupNoTag:(int32_t)fieldNumber value:(const GPBUnknownFieldSet *)value {
   [value writeToCodedOutputStream:self];
   GPBWriteTagWithFormat(&state_, fieldNumber, GPBWireFormatEndGroup);
@@ -343,6 +365,8 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
   [self writeUnknownGroupNoTag:fieldNumber value:value];
 }
 
+#pragma clang diagnostic pop
+
 - (void)writeMessageNoTag:(GPBMessage *)value {
   GPBWriteRawVarint32(&state_, (int32_t)[value serializedSize]);
   [value writeToCodedOutputStream:self];
@@ -819,8 +843,12 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
   }
 }
 
-//%PDDM-EXPAND WRITE_UNPACKABLE_DEFNS(UnknownGroup, GPBUnknownFieldSet)
-// This block of code is generated, do not edit it directly.
+//%PDDM-EXPAND-END (19 expansions)
+
+// clang-format on
+
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 
 - (void)writeUnknownGroupArray:(int32_t)fieldNumber values:(NSArray *)values {
   for (GPBUnknownFieldSet *value in values) {
@@ -828,9 +856,7 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
   }
 }
 
-//%PDDM-EXPAND-END (19 expansions)
-
-// clang-format on
+#pragma clang diagnostic pop
 
 - (void)writeMessageSetExtension:(int32_t)fieldNumber value:(GPBMessage *)value {
   GPBWriteTagWithFormat(&state_, GPBWireFormatMessageSetItem, GPBWireFormatStartGroup);
@@ -891,7 +917,7 @@ static void GPBWriteRawLittleEndian64(GPBOutputBufferState *state, int64_t value
       state_.position = length;
     } else {
       // Write is very big.  Let's do it all at once.
-      NSInteger written = [state_.output write:((uint8_t *)value) + offset maxLength:length];
+      NSInteger written = WriteToOutputStream(state_.output, ((uint8_t *)value) + offset, length);
       if (written != (NSInteger)length) {
         [NSException raise:GPBCodedOutputStreamException_WriteFailed format:@""];
       }
@@ -963,7 +989,10 @@ size_t GPBComputeStringSizeNoTag(NSString *value) {
 
 size_t GPBComputeGroupSizeNoTag(GPBMessage *value) { return [value serializedSize]; }
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 size_t GPBComputeUnknownGroupSizeNoTag(GPBUnknownFieldSet *value) { return value.serializedSize; }
+#pragma clang diagnostic pop
 
 size_t GPBComputeMessageSizeNoTag(GPBMessage *value) {
   size_t size = [value serializedSize];
@@ -1031,9 +1060,12 @@ size_t GPBComputeGroupSize(int32_t fieldNumber, GPBMessage *value) {
   return GPBComputeTagSize(fieldNumber) * 2 + GPBComputeGroupSizeNoTag(value);
 }
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 size_t GPBComputeUnknownGroupSize(int32_t fieldNumber, GPBUnknownFieldSet *value) {
   return GPBComputeTagSize(fieldNumber) * 2 + GPBComputeUnknownGroupSizeNoTag(value);
 }
+#pragma clang diagnostic pop
 
 size_t GPBComputeMessageSize(int32_t fieldNumber, GPBMessage *value) {
   return GPBComputeTagSize(fieldNumber) + GPBComputeMessageSizeNoTag(value);
