@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #import <Foundation/Foundation.h>
 
@@ -39,6 +16,7 @@
 @class GPBExtensionDescriptor;
 @class GPBFieldDescriptor;
 @class GPBUnknownFieldSet;
+@class GPBUnknownFields;
 
 NS_ASSUME_NONNULL_BEGIN
 
@@ -77,8 +55,8 @@ CF_EXTERN_C_END
  *       exist in two places, you don't want a sub message to be a property
  *       property of two other messages.
  *
- * @note While the class support NSSecureCoding, if the message has any
- *       extensions, they will end up reloaded in @c unknownFields as there is
+ * @note While the class supports NSSecureCoding, if the message has any
+ *       extensions, they will end up reloaded in the unknown fields as there is
  *       no way for the @c NSCoding plumbing to pass through a
  *       @c GPBExtensionRegistry. To support extensions, instead of passing the
  *       calls off to the Message, simple store the result of @c data, and then
@@ -94,11 +72,10 @@ CF_EXTERN_C_END
 
 /**
  * The set of unknown fields for this message.
- *
- * Only messages from proto files declared with "proto2" syntax support unknown
- * fields.
  **/
-@property(nonatomic, copy, nullable) GPBUnknownFieldSet *unknownFields;
+@property(nonatomic, copy, nullable) GPBUnknownFieldSet *unknownFields __attribute__((
+    deprecated("Use GPBUnknownFields and the -initFromMessage: initializer and "
+               "mergeUnknownFields:extensionRegistry:error: to add the data back to a message.")));
 
 /**
  * Whether the message, along with all submessages, have the required fields
@@ -519,6 +496,40 @@ CF_EXTERN_C_END
  **/
 - (void)clear;
 
+/**
+ * Clears any unknown fields on this message.
+ *
+ * Note: To clear this message's unknown field and all the unknown fields of the
+ * messages within the fields of this message, use
+ * `GPBMessageDropUnknownFieldsRecursively()`.
+ **/
+- (void)clearUnknownFields;
+
+/**
+ * Merges in the data from an `GPBUnknownFields`, meaning the data from the unknown fields gets
+ * re-parsed so any known fields will be propertly set.
+ *
+ * If the intent is to *replace* the message's unknown fields, call `-clearUnknownFields` first.
+ *
+ * Since the data from the GPBUnknownFields will always be well formed, this call will almost never
+ * fail. What could cause it to fail is if the GPBUnknownFields contains a field value that is
+ * an error for the message's schema - i.e.: if it contains a length delimited field where the
+ * field number for the message is defined to be a _string_ field, however the length delimited
+ * data provide is not a valid UTF8 string, or if the field is a _packed_ number field, but the
+ * data provided is not a valid for that field.
+ *
+ * @param unknownFields     The unknown fields to merge the data from.
+ * @param extensionRegistry The extension registry to use to look up extensions, can be `nil`.
+ * @param errorPtr          An optional error pointer to fill in with a failure
+ *                          reason if the data can not be parsed. Will only be
+ *                          filled in if the data failed to be parsed.
+ *
+ * @return Boolean indicating success. errorPtr will only be fill in on failure.
+ **/
+- (BOOL)mergeUnknownFields:(GPBUnknownFields *)unknownFields
+         extensionRegistry:(nullable id<GPBExtensionRegistry>)extensionRegistry
+                     error:(NSError **)errorPtr;
+
 @end
 
 NS_ASSUME_NONNULL_END
