@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2015 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #import "GPBTestUtilities.h"
 
@@ -46,7 +23,7 @@
 
 @implementation MessageSerializationTests
 
-// TODO(thomasvl): Pull tests over from GPBMessageTests that are serialization
+// TODO: Pull tests over from GPBMessageTests that are serialization
 // specific.
 
 - (void)testProto3SerializationHandlingDefaults {
@@ -442,6 +419,20 @@
 
   // All the values should be in unknown fields.
 
+  GPBUnknownFields *ufs = [[GPBUnknownFields alloc] initFromMessage:msg];
+  XCTAssertEqual(ufs.count, 3U);
+  uint64_t varint;
+  XCTAssertTrue([ufs getFirst:Message2_FieldNumber_OptionalEnum varint:&varint]);
+  XCTAssertEqual(varint, (uint64_t)Message3_Enum_Extra3);
+  XCTAssertTrue([ufs getFirst:Message2_FieldNumber_RepeatedEnumArray varint:&varint]);
+  XCTAssertEqual(varint, (uint64_t)Message3_Enum_Extra3);
+  XCTAssertTrue([ufs getFirst:Message2_FieldNumber_OneofEnum varint:&varint]);
+  XCTAssertEqual(varint, (uint64_t)Message3_Enum_Extra3);
+  [ufs release];
+
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+
   GPBUnknownFieldSet *unknownFields = msg.unknownFields;
 
   XCTAssertEqual([unknownFields countOfFields], 3U);
@@ -461,6 +452,8 @@
   XCTAssertEqual(field.varintList.count, 1U);
   XCTAssertEqual([field.varintList valueAtIndex:0], (uint64_t)Message3_Enum_Extra3);
 
+#pragma clang diagnostic pop
+
   [msg release];
   [orig release];
 }
@@ -1421,7 +1414,13 @@
   int32_t val = -1;
   XCTAssertTrue([msg1.knownMapField getEnum:&val forKey:0]);
   XCTAssertEqual(val, Proto2MapEnum_Proto2MapEnumFoo);
+  GPBUnknownFields *ufs = [[GPBUnknownFields alloc] initFromMessage:msg1];
+  XCTAssertEqual(ufs.count, 1U);
+  [ufs release];
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
   XCTAssertEqual(msg1.unknownFields.countOfFields, 1U);
+#pragma clang diagnostic pop
 
   data = [msg1 data];
   TestEnumMapPlusExtra *msg2 = [TestEnumMapPlusExtra parseFromData:data error:NULL];
@@ -1433,7 +1432,13 @@
   XCTAssertEqual(msg2.unknownMapField.count, 1U);
   XCTAssertTrue([msg2.unknownMapField getEnum:&val forKey:0]);
   XCTAssertEqual(val, Proto2MapEnumPlusExtra_EProto2MapEnumExtra);
+  ufs = [[GPBUnknownFields alloc] initFromMessage:msg2];
+  XCTAssertTrue(ufs.empty);
+  [ufs release];
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
   XCTAssertEqual(msg2.unknownFields.countOfFields, 0U);
+#pragma clang diagnostic pop
 
   XCTAssertEqualObjects(orig, msg2);
 
