@@ -1,14 +1,17 @@
 #!/usr/bin/ruby
 
 require 'google/protobuf'
+require 'repeated_field_test_pb'
 require 'test/unit'
 
 class RepeatedFieldTest < Test::Unit::TestCase
+  TestMessage = RepeatedFieldTestProtos::TestMessage
+  TestMessage2 = RepeatedFieldTestProtos::TestMessage2
 
   def test_acts_like_enumerator
     m = TestMessage.new
     (Enumerable.instance_methods - TestMessage.new.repeated_string.methods).each do |method_name|
-      assert m.repeated_string.respond_to?(method_name) == true, "does not respond to #{method_name}"
+      assert_respond_to m.repeated_string, method_name
     end
   end
 
@@ -19,11 +22,13 @@ class RepeatedFieldTest < Test::Unit::TestCase
     arr_methods -= [ :indices, :iter_for_each, :iter_for_each_index,
       :iter_for_each_with_index, :dimensions, :copy_data, :copy_data_simple,
       :nitems, :iter_for_reverse_each, :indexes, :append, :prepend]
-    arr_methods -= [:union, :difference, :filter!]
-    arr_methods -= [:intersection, :deconstruct] # ruby 2.7 methods we can ignore
-    arr_methods -= [:intersect?] # ruby 3.1 methods we can ignore
+    arr_methods -= [:filter!]
+    # ruby 2.7 methods we can ignore
+    arr_methods -= [:deconstruct, :resolve_feature_path]
+    # ruby 3.1 methods we can ignore
+    arr_methods -= [:intersect?]
     arr_methods.each do |method_name|
-      assert m.repeated_string.respond_to?(method_name) == true, "does not respond to #{method_name}"
+      assert_respond_to m.repeated_string, method_name
     end
   end
 
@@ -31,8 +36,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     m = TestMessage.new
     repeated_field_names(TestMessage).each do |field_name|
       assert_nil m.send(field_name).first
-      assert_equal [], m.send(field_name).first(0)
-      assert_equal [], m.send(field_name).first(1)
+      assert_empty m.send(field_name).first(0)
     end
 
     fill_test_msg(m)
@@ -40,7 +44,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal( -1_000_000, m.repeated_int64.first )
     assert_equal 10, m.repeated_uint32.first
     assert_equal 1_000_000, m.repeated_uint64.first
-    assert_equal true, m.repeated_bool.first
+    assert m.repeated_bool.first
     assert_equal( -1.01,  m.repeated_float.first.round(2) )
     assert_equal( -1.0000000000001, m.repeated_double.first )
     assert_equal 'foo', m.repeated_string.first
@@ -52,7 +56,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
       m.repeated_int32.first(-1)
     end
     assert_equal "negative array size", err.message
-    assert_equal [], m.repeated_int32.first(0)
+    assert_empty m.repeated_int32.first(0)
     assert_equal [-10], m.repeated_int32.first(1)
     assert_equal [-10, -11], m.repeated_int32.first(2)
     assert_equal [-10, -11], m.repeated_int32.first(3)
@@ -69,7 +73,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal( -1_000_001, m.repeated_int64.last )
     assert_equal 11, m.repeated_uint32.last
     assert_equal 1_000_001, m.repeated_uint64.last
-    assert_equal false, m.repeated_bool.last
+    refute m.repeated_bool.last
     assert_equal( -1.02, m.repeated_float.last.round(2) )
     assert_equal( -1.0000000000002, m.repeated_double.last )
     assert_equal 'bar', m.repeated_string.last
@@ -81,7 +85,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
       m.repeated_int32.last(-1)
     end
     assert_equal "negative array size", err.message
-    assert_equal [], m.repeated_int32.last(0)
+    assert_empty m.repeated_int32.last(0)
     assert_equal [-11], m.repeated_int32.last(1)
     assert_equal [-10, -11], m.repeated_int32.last(2)
     assert_equal [-10, -11], m.repeated_int32.last(3)
@@ -103,8 +107,8 @@ class RepeatedFieldTest < Test::Unit::TestCase
     assert_equal 10, m.repeated_uint32.pop
     assert_equal 1_000_001, m.repeated_uint64.pop
     assert_equal 1_000_000, m.repeated_uint64.pop
-    assert_equal false, m.repeated_bool.pop
-    assert_equal true, m.repeated_bool.pop
+    refute m.repeated_bool.pop
+    assert m.repeated_bool.pop
     assert_equal( -1.02,  m.repeated_float.pop.round(2) )
     assert_equal( -1.01,  m.repeated_float.pop.round(2) )
     assert_equal( -1.0000000000002, m.repeated_double.pop )
@@ -141,19 +145,33 @@ class RepeatedFieldTest < Test::Unit::TestCase
   end
 
 
+  def test_each_index
+    m = TestMessage.new
+    5.times{|i| m.repeated_string << 'string' }
+
+    expected = 0
+    m.repeated_string.each_index do |idx|
+      assert_equal expected, idx
+      expected += 1
+      assert_equal 'string', m.repeated_string[idx]
+    end
+    assert_equal 5, expected
+  end
+
+
   def test_empty?
     m = TestMessage.new
-    assert_equal true, m.repeated_string.empty?
+    assert_empty m.repeated_string
     m.repeated_string << 'foo'
-    assert_equal false, m.repeated_string.empty?
+    refute_empty m.repeated_string
     m.repeated_string << 'bar'
-    assert_equal false, m.repeated_string.empty?
+    refute_empty m.repeated_string
   end
 
   def test_reassign
     m = TestMessage.new
     m.repeated_msg = Google::Protobuf::RepeatedField.new(:message, TestMessage2, [TestMessage2.new(:foo => 1)])
-    assert_equal m.repeated_msg.first, TestMessage2.new(:foo => 1)
+    assert_equal TestMessage2.new(:foo => 1), m.repeated_msg.first
   end
 
   def test_array_accessor
@@ -265,7 +283,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     m.repeated_msg[3] = TestMessage2.new(:foo => 1)
     assert_equal [nil, nil, nil, TestMessage2.new(:foo => 1)], m.repeated_msg
     m.repeated_enum[3] = :A
-    assert_equal [:Default, :Default, :Default, :A], m.repeated_enum
+    assert_equal [:DEFAULT, :DEFAULT, :DEFAULT, :A], m.repeated_enum
 
     # check_self_modifying_method(m.repeated_string, reference_arr) do |arr|
     #   arr[20] = 'spacious'
@@ -323,7 +341,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     m.repeated_string += reference_arr.clone
     assert_equal reference_arr, m.repeated_string
     reference_arr << 'fizz'
-    assert_not_equal reference_arr, m.repeated_string
+    refute_equal reference_arr, m.repeated_string
     m.repeated_string << 'fizz'
     assert_equal reference_arr, m.repeated_string
   end
@@ -337,7 +355,7 @@ class RepeatedFieldTest < Test::Unit::TestCase
     hash = m.repeated_string.hash
     assert_equal hash, m.repeated_string.hash
     m.repeated_string << 'j'
-    assert_not_equal hash, m.repeated_string.hash
+    refute_equal hash, m.repeated_string.hash
   end
 
   def test_plus
@@ -658,52 +676,4 @@ class RepeatedFieldTest < Test::Unit::TestCase
     test_msg.repeated_enum   << :A
     test_msg.repeated_enum   << :B
   end
-
-
-  pool = Google::Protobuf::DescriptorPool.new
-  pool.build do
-
-    add_message "TestMessage" do
-      optional :optional_int32,  :int32,        1
-      optional :optional_int64,  :int64,        2
-      optional :optional_uint32, :uint32,       3
-      optional :optional_uint64, :uint64,       4
-      optional :optional_bool,   :bool,         5
-      optional :optional_float,  :float,        6
-      optional :optional_double, :double,       7
-      optional :optional_string, :string,       8
-      optional :optional_bytes,  :bytes,        9
-      optional :optional_msg,    :message,      10, "TestMessage2"
-      optional :optional_enum,   :enum,         11, "TestEnum"
-
-      repeated :repeated_int32,  :int32,        12
-      repeated :repeated_int64,  :int64,        13
-      repeated :repeated_uint32, :uint32,       14
-      repeated :repeated_uint64, :uint64,       15
-      repeated :repeated_bool,   :bool,         16
-      repeated :repeated_float,  :float,        17
-      repeated :repeated_double, :double,       18
-      repeated :repeated_string, :string,       19
-      repeated :repeated_bytes,  :bytes,        20
-      repeated :repeated_msg,    :message,      21, "TestMessage2"
-      repeated :repeated_enum,   :enum,         22, "TestEnum"
-    end
-    add_message "TestMessage2" do
-      optional :foo, :int32, 1
-    end
-
-    add_enum "TestEnum" do
-      value :Default, 0
-      value :A, 1
-      value :B, 2
-      value :C, 3
-      value :v0, 4
-    end
-  end
-
-  TestMessage = pool.lookup("TestMessage").msgclass
-  TestMessage2 = pool.lookup("TestMessage2").msgclass
-  TestEnum = pool.lookup("TestEnum").enummodule
-
-
 end
