@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
@@ -35,19 +12,22 @@
 #include "google/protobuf/compiler/cpp/enum.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <limits>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "google/protobuf/descriptor.h"
+#include "absl/algorithm/container.h"
 #include "absl/container/btree_map.h"
 #include "absl/container/btree_set.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/strings/str_cat.h"
 #include "google/protobuf/compiler/cpp/helpers.h"
 #include "google/protobuf/compiler/cpp/names.h"
+#include "google/protobuf/descriptor.h"
+#include "google/protobuf/generated_enum_util.h"
 
 namespace google {
 namespace protobuf {
@@ -61,7 +41,7 @@ absl::flat_hash_map<absl::string_view, std::string> EnumVars(
     const EnumValueDescriptor* min, const EnumValueDescriptor* max) {
   auto classname = ClassName(enum_, false);
   return {
-      {"Enum", enum_->name()},
+      {"Enum", std::string(enum_->name())},
       {"Enum_", ResolveKeyword(enum_->name())},
       {"Msg_Enum", classname},
       {"::Msg_Enum", QualifiedClassName(enum_, options)},
@@ -142,9 +122,8 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
                                         EnumValueName(value)))
                            .AnnotatedAs(value),
                        {"kNumber", Int32ToString(value->number())},
-                       {"DEPRECATED", value->options().deprecated()
-                                          ? "PROTOBUF_DEPRECATED_ENUM"
-                                          : ""},
+                       {"DEPRECATED",
+                        value->options().deprecated() ? "[[deprecated]]" : ""},
                    },
                    R"cc(
                      $Msg_Enum_VALUE$$ DEPRECATED$ = $kNumber$,
@@ -181,6 +160,7 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
         };
 
         $dllexport_decl $bool $Msg_Enum$_IsValid(int value);
+        $dllexport_decl $extern const uint32_t $Msg_Enum$_internal_data_[];
         constexpr $Msg_Enum$ $Msg_Enum_Enum_MIN$ = static_cast<$Msg_Enum$>($kMin$);
         constexpr $Msg_Enum$ $Msg_Enum_Enum_MAX$ = static_cast<$Msg_Enum$>($kMax$);
       )cc");
@@ -242,10 +222,6 @@ void EnumGenerator::GenerateDefinition(io::Printer* p) {
               static_cast<int>(value));
         }
       )cc");
-    } else {
-      p->Emit(R"cc(
-        const std::string& $Msg_Enum$_Name($Msg_Enum$ value);
-      )cc");
     }
   } else {
     p->Emit({{"static_assert", write_assert}}, R"cc(
@@ -289,6 +265,7 @@ void EnumGenerator::GenerateGetEnumDescriptorSpecializations(io::Printer* p) {
   )cc");
 }
 
+
 void EnumGenerator::GenerateSymbolImports(io::Printer* p) const {
   auto v = p->WithVars(EnumVars(enum_, options_, limits_.min, limits_.max));
 
@@ -302,7 +279,7 @@ void EnumGenerator::GenerateSymbolImports(io::Printer* p) const {
         {
             Sub("VALUE", EnumValueName(enum_->value(j))).AnnotatedAs(value),
             {"DEPRECATED",
-             value->options().deprecated() ? "PROTOBUF_DEPRECATED_ENUM" : ""},
+             value->options().deprecated() ? "[[deprecated]]" : ""},
         },
         R"cc(
           $DEPRECATED $static constexpr $Enum_$ $VALUE$ = $Msg_Enum$_$VALUE$;
@@ -366,39 +343,65 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
     )cc");
   }
 
-  p->Emit({{"cases",
+  // Multiple values may have the same number. Sort and dedup.
+  std::vector<int> numbers;
+  numbers.reserve(enum_->value_count());
+  for (int i = 0; i < enum_->value_count(); ++i) {
+    numbers.push_back(enum_->value(i)->number());
+  }
+  // Sort and deduplicate `numbers`.
+  absl::c_sort(numbers);
+  numbers.erase(std::unique(numbers.begin(), numbers.end()), numbers.end());
+
+  // We now generate the XXX_IsValid functions, as well as their encoded enum
+  // data.
+  // For simple enums we skip the generic ValidateEnum call and use better
+  // codegen. It matches the speed of the previous switch-based codegen.
+  // For more complex enums we use the new algorithm with the encoded data.
+  // Always generate the data array, even on the simple cases because someone
+  // might be using it for TDP entries. If it is not used in the end, the linker
+  // will drop it.
+  p->Emit({{"encoded",
             [&] {
-              // Multiple values may have the same number.  Make sure we only
-              // cover each number once by first constructing a set containing
-              // all valid numbers, then printing a case statement for each
-              // element.
-
-              std::vector<int> numbers;
-              numbers.reserve(enum_->value_count());
-              for (int i = 0; i < enum_->value_count(); ++i) {
-                numbers.push_back(enum_->value(i)->number());
-              }
-              // Sort and deduplicate `numbers`.
-              absl::c_sort(numbers);
-              numbers.erase(std::unique(numbers.begin(), numbers.end()),
-                            numbers.end());
-
-              for (int n : numbers) {
-                p->Emit({{"n", n}}, R"cc(
-                  case $n$:
-                )cc");
+              for (uint32_t n : google::protobuf::internal::GenerateEnumData(numbers)) {
+                p->Emit({{"n", n}}, "$n$u, ");
               }
             }}},
-          R"(
-            bool $Msg_Enum$_IsValid(int value) {
-              switch (value) {
-                $cases$;
-                  return true;
-                default:
-                  return false;
+          R"cc(
+            PROTOBUF_CONSTINIT const uint32_t $Msg_Enum$_internal_data_[] = {
+                $encoded$};
+          )cc");
+
+  if (numbers.front() + static_cast<int64_t>(numbers.size()) - 1 ==
+      numbers.back()) {
+    // They are sequential. Do a simple range check.
+    p->Emit({{"min", numbers.front()}, {"max", numbers.back()}},
+            R"cc(
+              bool $Msg_Enum$_IsValid(int value) {
+                return $min$ <= value && value <= $max$;
               }
-            }
-          )");
+            )cc");
+  } else if (numbers.front() >= 0 && numbers.back() < 64) {
+    // Not sequential, but they fit in a 64-bit bitmap.
+    uint64_t bitmap = 0;
+    for (int n : numbers) {
+      bitmap |= uint64_t{1} << n;
+    }
+    p->Emit({{"bitmap", bitmap}, {"max", numbers.back()}},
+            R"cc(
+              bool $Msg_Enum$_IsValid(int value) {
+                return 0 <= value && value <= $max$ && (($bitmap$u >> value) & 1) != 0;
+              }
+            )cc");
+  } else {
+    // More complex struct. Use enum data structure for lookup.
+    p->Emit(
+        R"cc(
+          bool $Msg_Enum$_IsValid(int value) {
+            return ::_pbi::ValidateEnum(value, $Msg_Enum$_internal_data_);
+          }
+        )cc");
+  }
 
   if (!has_reflection_) {
     // In lite mode (where descriptors are unavailable), we generate separate
@@ -439,12 +442,12 @@ void EnumGenerator::GenerateMethods(int idx, io::Printer* p) {
       ++index;
       offset += e.first.size();
     }
-    absl::c_sort(offsets, [](const auto& a, const auto& b) {
+    absl::c_stable_sort(offsets, [](const auto& a, const auto& b) {
       return a.byte_offset < b.byte_offset;
     });
 
     std::vector<Offset> offsets_by_number = offsets;
-    absl::c_sort(offsets_by_number, [](const auto& a, const auto& b) {
+    absl::c_stable_sort(offsets_by_number, [](const auto& a, const auto& b) {
       return a.number < b.number;
     });
 
