@@ -20,7 +20,6 @@
 #include <string>
 #include <utility>
 
-#include "absl/base/dynamic_annotations.h"
 #include "absl/log/absl_check.h"
 #include "absl/log/absl_log.h"
 #include "absl/strings/cord.h"
@@ -28,13 +27,15 @@
 #include "absl/strings/internal/resize_uninitialized.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
-#include "absl/synchronization/mutex.h"
 #include "absl/types/optional.h"
+#include "absl/types/span.h"
 #include "google/protobuf/arena.h"
+#include "google/protobuf/generated_message_tctable_impl.h"
 #include "google/protobuf/io/coded_stream.h"
 #include "google/protobuf/io/zero_copy_stream.h"
 #include "google/protobuf/io/zero_copy_stream_impl.h"
 #include "google/protobuf/io/zero_copy_stream_impl_lite.h"
+#include "google/protobuf/metadata_lite.h"
 #include "google/protobuf/parse_context.h"
 
 
@@ -44,17 +45,64 @@
 namespace google {
 namespace protobuf {
 
-std::string MessageLite::GetTypeName() const {
-  auto* data = GetClassData();
-  ABSL_DCHECK(data != nullptr);
+void MessageLite::DestroyInstance(bool free_memory) {
+#if defined(PROTOBUF_CUSTOM_VTABLE)
+  _class_data_->delete_message(this, free_memory);
+#else   // PROTOBUF_CUSTOM_VTABLE
+  ABSL_DCHECK(!free_memory);
+  this->~MessageLite();
+#endif  // PROTOBUF_CUSTOM_VTABLE
+}
 
-  if (!data->is_lite) {
+void MessageLite::CheckTypeAndMergeFrom(const MessageLite& other) {
+  auto* data = GetClassData();
+  auto* other_data = other.GetClassData();
+
+  ABSL_CHECK_EQ(data, other_data)
+      << "Invalid call to CheckTypeAndMergeFrom between types " << GetTypeName()
+      << " and " << other.GetTypeName();
+  data->merge_to_from(*this, other);
+}
+
+#if defined(PROTOBUF_CUSTOM_VTABLE)
+uint8_t* MessageLite::_InternalSerialize(
+    uint8_t* ptr, io::EpsCopyOutputStream* stream) const {
+  return _class_data_->serialize(*this, ptr, stream);
+}
+
+MessageLite* MessageLite::New(Arena* arena) const {
+  return static_cast<MessageLite*>(_class_data_->new_message(this, arena));
+}
+
+void MessageLite::Clear() { _class_data_->clear(*this); }
+
+size_t MessageLite::ByteSizeLong() const {
+  return _class_data_->byte_size_long(*this);
+}
+#endif  // PROTOBUF_CUSTOM_VTABLE
+
+bool MessageLite::IsInitialized() const {
+  auto* data = GetClassData();
+  return data->is_initialized != nullptr ? data->is_initialized(*this) : true;
+}
+
+const char* MessageLite::_InternalParse(const char* ptr,
+                                        internal::ParseContext* ctx) {
+  return internal::TcParser::ParseLoop(this, ptr, ctx, GetTcParseTable());
+}
+
+std::string MessageLite::GetTypeName() const {
+  return std::string(TypeId::Get(*this).name());
+}
+
+absl::string_view TypeId::name() const {
+  if (!data_->is_lite) {
     // For !LITE messages, we use the descriptor method function.
-    return data->full().descriptor_methods->get_type_name(*this);
+    return data_->full().descriptor_methods->get_type_name(data_);
   }
 
   // For LITE messages, the type name is a char[] just beyond ClassData.
-  return reinterpret_cast<const char*>(data) + sizeof(ClassData);
+  return reinterpret_cast<const char*>(data_) + sizeof(MessageLite::ClassData);
 }
 
 void MessageLite::OnDemandRegisterArenaDtor(Arena* arena) {
@@ -80,6 +128,12 @@ std::string MessageLite::InitializationErrorString() const {
 }
 
 std::string MessageLite::DebugString() const {
+  auto* data = GetClassData();
+  ABSL_DCHECK(data != nullptr);
+  if (!data->is_lite) {
+    return data->full().descriptor_methods->debug_string(*this);
+  }
+
   return absl::StrCat("MessageLite at 0x", absl::Hex(this));
 }
 
@@ -147,13 +201,25 @@ void MessageLite::LogInitializationErrorMessage() const {
 
 namespace internal {
 
+void FailDynamicCast(const MessageLite& from, const MessageLite& to) {
+  const auto to_name = to.GetTypeName();
+  if (internal::GetClassData(from)->is_dynamic) {
+    ABSL_LOG(FATAL)
+        << "Cannot downcast from a DynamicMessage to generated type "
+        << to_name;
+  }
+  const auto from_name = from.GetTypeName();
+  ABSL_LOG(FATAL) << "Cannot downcast " << from_name << " to " << to_name;
+}
+
 template <bool aliasing>
 bool MergeFromImpl(absl::string_view input, MessageLite* msg,
+                   const internal::TcParseTableBase* tc_table,
                    MessageLite::ParseFlags parse_flags) {
   const char* ptr;
   internal::ParseContext ctx(io::CodedInputStream::GetDefaultRecursionLimit(),
                              aliasing, &ptr, input);
-  ptr = msg->_InternalParse(ptr, &ctx);
+  ptr = internal::TcParser::ParseLoop(msg, ptr, &ctx, tc_table);
   // ctx has an explicit limit set (length of string_view).
   if (PROTOBUF_PREDICT_TRUE(ptr && ctx.EndedAtLimit())) {
     return CheckFieldPresence(ctx, *msg, parse_flags);
@@ -163,11 +229,12 @@ bool MergeFromImpl(absl::string_view input, MessageLite* msg,
 
 template <bool aliasing>
 bool MergeFromImpl(io::ZeroCopyInputStream* input, MessageLite* msg,
+                   const internal::TcParseTableBase* tc_table,
                    MessageLite::ParseFlags parse_flags) {
   const char* ptr;
   internal::ParseContext ctx(io::CodedInputStream::GetDefaultRecursionLimit(),
                              aliasing, &ptr, input);
-  ptr = msg->_InternalParse(ptr, &ctx);
+  ptr = internal::TcParser::ParseLoop(msg, ptr, &ctx, tc_table);
   // ctx has no explicit limit (hence we end on end of stream)
   if (PROTOBUF_PREDICT_TRUE(ptr && ctx.EndedAtEndOfStream())) {
     return CheckFieldPresence(ctx, *msg, parse_flags);
@@ -177,11 +244,12 @@ bool MergeFromImpl(io::ZeroCopyInputStream* input, MessageLite* msg,
 
 template <bool aliasing>
 bool MergeFromImpl(BoundedZCIS input, MessageLite* msg,
+                   const internal::TcParseTableBase* tc_table,
                    MessageLite::ParseFlags parse_flags) {
   const char* ptr;
   internal::ParseContext ctx(io::CodedInputStream::GetDefaultRecursionLimit(),
                              aliasing, &ptr, input.zcis, input.limit);
-  ptr = msg->_InternalParse(ptr, &ctx);
+  ptr = internal::TcParser::ParseLoop(msg, ptr, &ctx, tc_table);
   if (PROTOBUF_PREDICT_FALSE(!ptr)) return false;
   ctx.BackUp(ptr);
   if (PROTOBUF_PREDICT_TRUE(ctx.EndedAtLimit())) {
@@ -191,18 +259,24 @@ bool MergeFromImpl(BoundedZCIS input, MessageLite* msg,
 }
 
 template bool MergeFromImpl<false>(absl::string_view input, MessageLite* msg,
+                                   const internal::TcParseTableBase* tc_table,
                                    MessageLite::ParseFlags parse_flags);
 template bool MergeFromImpl<true>(absl::string_view input, MessageLite* msg,
+                                  const internal::TcParseTableBase* tc_table,
                                   MessageLite::ParseFlags parse_flags);
 template bool MergeFromImpl<false>(io::ZeroCopyInputStream* input,
                                    MessageLite* msg,
+                                   const internal::TcParseTableBase* tc_table,
                                    MessageLite::ParseFlags parse_flags);
 template bool MergeFromImpl<true>(io::ZeroCopyInputStream* input,
                                   MessageLite* msg,
+                                  const internal::TcParseTableBase* tc_table,
                                   MessageLite::ParseFlags parse_flags);
 template bool MergeFromImpl<false>(BoundedZCIS input, MessageLite* msg,
+                                   const internal::TcParseTableBase* tc_table,
                                    MessageLite::ParseFlags parse_flags);
 template bool MergeFromImpl<true>(BoundedZCIS input, MessageLite* msg,
+                                  const internal::TcParseTableBase* tc_table,
                                   MessageLite::ParseFlags parse_flags);
 
 }  // namespace internal
@@ -247,7 +321,7 @@ bool MessageLite::MergeFromImpl(io::CodedInputStream* input,
   ctx.TrackCorrectEnding();
   ctx.data().pool = input->GetExtensionPool();
   ctx.data().factory = input->GetExtensionFactory();
-  ptr = _InternalParse(ptr, &ctx);
+  ptr = internal::TcParser::ParseLoop(this, ptr, &ctx, GetTcParseTable());
   if (PROTOBUF_PREDICT_FALSE(!ptr)) return false;
   ctx.BackUp(ptr);
   if (!ctx.EndedAtEndOfStream()) {
@@ -354,13 +428,14 @@ template <>
 struct SourceWrapper<absl::Cord> {
   explicit SourceWrapper(const absl::Cord* c) : cord(c) {}
   template <bool alias>
-  bool MergeInto(MessageLite* msg, MessageLite::ParseFlags parse_flags) const {
+  bool MergeInto(MessageLite* msg, const internal::TcParseTableBase* tc_table,
+                 MessageLite::ParseFlags parse_flags) const {
     absl::optional<absl::string_view> flat = cord->TryFlat();
     if (flat && flat->size() <= ParseContext::kMaxCordBytesToCopy) {
-      return MergeFromImpl<alias>(*flat, msg, parse_flags);
+      return MergeFromImpl<alias>(*flat, msg, tc_table, parse_flags);
     } else {
       io::CordInputStream input(cord);
-      return MergeFromImpl<alias>(&input, msg, parse_flags);
+      return MergeFromImpl<alias>(&input, msg, tc_table, parse_flags);
     }
   }
 
