@@ -1,43 +1,18 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
 //  Sanjay Ghemawat, Jeff Dean, and others.
 
-#include <google/protobuf/test_util_lite.h>
+#include "google/protobuf/test_util_lite.h"
 
-#include <google/protobuf/stubs/logging.h>
-#include <google/protobuf/stubs/common.h>
 #include <gtest/gtest.h>
-#include <google/protobuf/stubs/strutil.h>
+#include "absl/strings/string_view.h"
 
 namespace google {
 namespace protobuf {
@@ -326,7 +301,9 @@ void TestUtilLite::ExpectAllFieldsSet(
   EXPECT_EQ(310, message.repeated_sfixed64(1));
   EXPECT_EQ(311, message.repeated_float(1));
   EXPECT_EQ(312, message.repeated_double(1));
-  EXPECT_EQ(false, message.repeated_bool(1));
+  // EXPECT_EQ(false, ...) triggers a compiler warning in some platforms.
+  //   warning: converting ‘false’ to pointer type
+  EXPECT_FALSE(message.repeated_bool(1));
   EXPECT_EQ("315", message.repeated_string(1));
   EXPECT_EQ("316", message.repeated_bytes(1));
 
@@ -376,7 +353,7 @@ void TestUtilLite::ExpectAllFieldsSet(
   EXPECT_EQ(410, message.default_sfixed64());
   EXPECT_EQ(411, message.default_float());
   EXPECT_EQ(412, message.default_double());
-  EXPECT_EQ(false, message.default_bool());
+  EXPECT_FALSE(message.default_bool());
   EXPECT_EQ("415", message.default_string());
   EXPECT_EQ("416", message.default_bytes());
 
@@ -439,7 +416,7 @@ void TestUtilLite::ExpectClear(const unittest::TestAllTypesLite& message) {
   EXPECT_EQ(0, message.optional_sfixed64());
   EXPECT_EQ(0, message.optional_float());
   EXPECT_EQ(0, message.optional_double());
-  EXPECT_EQ(false, message.optional_bool());
+  EXPECT_FALSE(message.optional_bool());
   EXPECT_EQ("", message.optional_string());
   EXPECT_EQ("", message.optional_bytes());
 
@@ -728,7 +705,7 @@ void TestUtilLite::ExpectPackedFieldsSet(
   EXPECT_EQ(710, message.packed_sfixed64(1));
   EXPECT_EQ(711, message.packed_float(1));
   EXPECT_EQ(712, message.packed_double(1));
-  EXPECT_EQ(false, message.packed_bool(1));
+  EXPECT_FALSE(message.packed_bool(1));
   EXPECT_EQ(unittest::FOREIGN_LITE_BAZ, message.packed_enum(1));
 }
 
@@ -1256,8 +1233,7 @@ void TestUtilLite::ExpectAllExtensionsSet(
             message.GetExtension(unittest::repeated_float_extension_lite, 1));
   EXPECT_EQ(312,
             message.GetExtension(unittest::repeated_double_extension_lite, 1));
-  EXPECT_EQ(false,
-            message.GetExtension(unittest::repeated_bool_extension_lite, 1));
+  EXPECT_FALSE(message.GetExtension(unittest::repeated_bool_extension_lite, 1));
   EXPECT_EQ("315",
             message.GetExtension(unittest::repeated_string_extension_lite, 1));
   EXPECT_EQ("316",
@@ -1335,7 +1311,7 @@ void TestUtilLite::ExpectAllExtensionsSet(
             message.GetExtension(unittest::default_sfixed64_extension_lite));
   EXPECT_EQ(411, message.GetExtension(unittest::default_float_extension_lite));
   EXPECT_EQ(412, message.GetExtension(unittest::default_double_extension_lite));
-  EXPECT_EQ(false, message.GetExtension(unittest::default_bool_extension_lite));
+  EXPECT_FALSE(message.GetExtension(unittest::default_bool_extension_lite));
   EXPECT_EQ("415",
             message.GetExtension(unittest::default_string_extension_lite));
   EXPECT_EQ("416",
@@ -1428,8 +1404,7 @@ void TestUtilLite::ExpectExtensionsClear(
             message.GetExtension(unittest::optional_sfixed64_extension_lite));
   EXPECT_EQ(0, message.GetExtension(unittest::optional_float_extension_lite));
   EXPECT_EQ(0, message.GetExtension(unittest::optional_double_extension_lite));
-  EXPECT_EQ(false,
-            message.GetExtension(unittest::optional_bool_extension_lite));
+  EXPECT_FALSE(message.GetExtension(unittest::optional_bool_extension_lite));
   EXPECT_EQ("", message.GetExtension(unittest::optional_string_extension_lite));
   EXPECT_EQ("", message.GetExtension(unittest::optional_bytes_extension_lite));
 
@@ -1883,8 +1858,7 @@ void TestUtilLite::ExpectPackedExtensionsSet(
             message.GetExtension(unittest::packed_float_extension_lite, 1));
   EXPECT_EQ(712,
             message.GetExtension(unittest::packed_double_extension_lite, 1));
-  EXPECT_EQ(false,
-            message.GetExtension(unittest::packed_bool_extension_lite, 1));
+  EXPECT_FALSE(message.GetExtension(unittest::packed_bool_extension_lite, 1));
   EXPECT_EQ(unittest::FOREIGN_LITE_BAZ,
             message.GetExtension(unittest::packed_enum_extension_lite, 1));
 }
