@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #include "google/protobuf/util/type_resolver_util.h"
 
@@ -446,7 +423,7 @@ class DescriptorPoolTypeResolverSyntaxTest : public testing::Test {
 
   const FileDescriptor* BuildFile(
       absl::string_view syntax,
-      absl::optional<absl::string_view> edition = absl::nullopt) {
+      absl::optional<Edition> edition = absl::nullopt) {
     FileDescriptorProto proto;
     proto.set_package("test");
     proto.set_name("foo");
@@ -467,8 +444,7 @@ class DescriptorPoolTypeResolverSyntaxTest : public testing::Test {
 
 TEST_F(DescriptorPoolTypeResolverSyntaxTest, SyntaxProto2) {
   const FileDescriptor* file = BuildFile("proto2");
-  ASSERT_EQ(FileDescriptorLegacy::Syntax::SYNTAX_PROTO2,
-            FileDescriptorLegacy(file).syntax());
+  ASSERT_EQ(FileDescriptorLegacy(file).edition(), Edition::EDITION_PROTO2);
 
   Type type;
   ASSERT_TRUE(
@@ -479,14 +455,12 @@ TEST_F(DescriptorPoolTypeResolverSyntaxTest, SyntaxProto2) {
 
 TEST_F(DescriptorPoolTypeResolverSyntaxTest, SyntaxProto3) {
   const FileDescriptor* file = BuildFile("proto3");
-  ASSERT_EQ(FileDescriptorLegacy::Syntax::SYNTAX_PROTO3,
-            FileDescriptorLegacy(file).syntax());
+  ASSERT_EQ(FileDescriptorLegacy(file).edition(), Edition::EDITION_PROTO3);
 
   Type type;
   ASSERT_TRUE(
       resolver_->ResolveMessageType(GetTypeUrl("test.MyMessage"), &type).ok());
-  // TODO(b/271206501) This should be proto3.
-  EXPECT_EQ(type.syntax(), Syntax::SYNTAX_PROTO2);
+  EXPECT_EQ(type.syntax(), Syntax::SYNTAX_PROTO3);
   EXPECT_EQ(type.edition(), "");
 }
 
